 // Add C includes here

#if defined __cplusplus
 // Add C++ includes here
#include <iostream>
#include <QGroupBox>
#include <QFrame>
#include <QGridLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QListView>
#include <QTextEdit>
#include <QList>
#include <QTableView>
#include <QTextBrowser>
#include <QList>
#include <QVBoxLayout>
#include <qapplication.h>
#include <QCalendarWidget>
#include <QCheckBox>
#include <qcheckbox.h>
#include <QCloseEvent>
#include <qcolor.h>
#include <qcombobox.h>
#include <QDate>
#include <QDateTime>
#include <QDateTimeEdit>
#include <qdatetime.h>
#include <qdialog.h>
#include <qdir.h>
#include <QDragEnterEvent>
#include <QDragLeaveEvent>
#include <QDragMoveEvent>
#include <QDropEvent>
#include <QEvent>
#include <qevent.h>
#include <QFileDialog>
#include <qfiledialog.h>
#include <qfile.h>
#include <qfile.h>
#include <QFileInfo>
#include <qfileinfo.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qimage.h>
#include <QKeyEvent>
#include <QLabel>
#include <qlabel.h>
#include <QLatin1String>
#include <QLayout>
#include <qlayout.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <QList>
#include <qlist.h>
#include <QLocale>
#include <qmenubar.h>
#include <QMessageBox>
#include <qmessagebox.h>
#include <qmime.h>
#include <QMouseEvent>
#include <qobject.h>
#include <qpainter.h>
#include <QPaintEvent>
#include <qpalette.h>
#include <QPixmap>
#include <qpixmap.h>
#include <QPlastiqueStyle>
#include <qpoint.h>
#include <QProgressDialog>
#include <qprogressdialog.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <QRegExp>
#include <qregexp.h>
#include <QResizeEvent>
#include <QSpinBox>
#include <qspinbox.h>
#include <qsplitter.h>
#include <QString>
#include <qstring.h>
#include <QStringList>
#include <qstringlist.h>
#include <QFrame>
#include <QTextEdit>
#include <qtabwidget.h>
#include <QtCore/QVariant>
#include <QtDebug>
#include <QTextBrowser>
#include <qtextbrowser.h>
#include <qtextcodec.h>
#include <QTextStream>
#include <qtextstream.h>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QDateTimeEdit>
#include <QtGui/QDialog>
#include <QtGui/QPushButton>
#include <QtGui/QVBoxLayout>
#include <qtimer.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qtranslator.h>
#include <QtXml/QXmlAttributes>
#include <QtXml/QXmlDefaultHandler>
#include <QtXml/QXmlInputSource>
#include <QtXml/QXmlLocator>
#include <qvariant.h>
#include <qwidget.h>
#endif